/*
 WORD CLOCK - 8x8 NeoPixel Desktop Edition
 Originally by Andy Doro, modifications for setting time and
 other settings by https://www.marmosetelectronics.com/word-clock/

 See https://www.marmosetelectronics.com/word-clock/ for custom PCB and 3d printed case
 
 Hardware:
 
 - Arduino UNI, Arduino Nano or 5v Pro Mini, Custom board
 - DS3231 RTC
 - Adafruit NeoPixel NeoMatrix 8x8 or equiv


Software:

This code requires the following libraries:

 - RTClib https://github.com/adafruit/RTClib
 - Adafruit_GFX https://github.com/adafruit/Adafruit-GFX-Library
 - Adafruit_NeoPixel https://github.com/adafruit/Adafruit_NeoPixel
 - Adafruit_NeoMatrix https://github.com/adafruit/Adafruit_NeoMatrix
 - Bounce2 https://github.com/thomasfredericks/Bounce2

Wiring:

Arduino A5 (SCL) to SCL Pin 16 (SCL) DS3231
Arduino A4 (SDA) to SDA pin 15 (SDA) DS3221

Pushbutton between Arduino D7 and ground
Pushbutton between Arduino D8 and ground

8x8 Neopixel display to D6

4.7k pullup on SCL and SDA
 
 

 Aword clock using NeoPixel RGB LEDs for a color shift effect.
 
 grid pattern

 A T W E N T Y D
 Q U A R T E R Y
 F I V E H A L F
 D P A S T O R O
 F I V E I G H T
 S I X T H R E E
 T W E L E V E N
 F O U R N I N E
 

 
 Acknowledgements:
  - Thanks Dano for faceplate / 3D models & project inspiration!
 
 */


// include the library code:
// #include <SPI.h>
#include <Wire.h>
#include <RTClib.h>
// #include <RTC_DS3234.h>
#include <Adafruit_GFX.h>
#include <Adafruit_NeoMatrix.h>
#include <Adafruit_NeoPixel.h>
#define BOUNCE_WITH_PROMPT_DETECTION
#include <Bounce2.h>
#include <EEPROM.h>
// define how to write each of the words

// 64-bits is too big! let's use two 32-bit numbers to create masks.
unsigned long topMask;
unsigned long bottomMask;

// define masks for each word. we add them with "bitwise or" to generate a mask for the entire "phrase".
// have to use two different 32-bits numbers, luckily none of the words are spread across top and bottom halves of the NeoMatrix.
#define MFIVE topMask |= 0xF000        // these are in hexadecimal
#define MTEN topMask |= 0x58000000
#define AQUARTER topMask |= 0x80FE0000
#define TWENTY topMask |= 0x7E000000
#define HALF topMask |= 0xF00
#define PAST topMask |= 0x78
#define TO topMask |= 0xC
#define ONE bottomMask |= 0x43
#define TWO bottomMask |= 0xC040
#define THREE bottomMask |= 0x1F0000
#define FOUR bottomMask |= 0xF0
#define FIVE bottomMask |= 0xF0000000
#define SIX bottomMask |= 0xE00000
#define SEVEN bottomMask |= 0x800F00
#define EIGHT bottomMask |= 0x1F000000
#define NINE bottomMask |= 0xF
#define TEN bottomMask |= 0x1010100
#define ELEVEN bottomMask |= 0x3F00
#define TWELVE bottomMask |= 0xF600
#define ANDYDORO topMask |= 0x89010087

// Hours for setting
#define SETHOUR1 topMask |= 0x00800000
#define SETHOUR2 topMask |= 0x80800000
#define SETHOUR3 topMask |= 0x80C00000
#define SETHOUR4 topMask |= 0xC0C00000
#define SETHOUR5 topMask |= 0xC0D00000
#define SETHOUR6 topMask |= 0xD0D00000
#define SETHOUR7 topMask |= 0xD0D80000
#define SETHOUR8 topMask |= 0xD8D80000
#define SETHOUR9 topMask |= 0xD8DA0000
#define SETHOUR10 topMask |= 0xDADA0000
#define SETHOUR11 topMask |= 0xDADB0000
#define SETHOUR12 topMask |= 0xDBDB0000

// Show an A while in AM, nothing for PM
#define SETHOURAM topMask |= 0x00000400
#define SETHOURPM topMask |= 0x00000000

// 7 segment digits for minute setting
#define SETMIN00 topMask |= 0x000000E0; bottomMask |= 0xA0A0A0E0
#define SETMIN10 topMask |= 0x00000020; bottomMask |= 0x20202020
#define SETMIN20 topMask |= 0x000000E0; bottomMask |= 0x20E080E0
#define SETMIN30 topMask |= 0x000000E0; bottomMask |= 0x20E020E0
#define SETMIN40 topMask |= 0x000000A0; bottomMask |= 0xA0E02020
#define SETMIN50 topMask |= 0x000000E0; bottomMask |= 0x80E020E0

#define SETMIN0 topMask |= 0x00000007; bottomMask |= 0x05050507
#define SETMIN1 topMask |= 0x00000001; bottomMask |= 0x01010101
#define SETMIN2 topMask |= 0x00000007; bottomMask |= 0x01070407
#define SETMIN3 topMask |= 0x00000007; bottomMask |= 0x01070107
#define SETMIN4 topMask |= 0x00000005; bottomMask |= 0x05070101
#define SETMIN5 topMask |= 0x00000007; bottomMask |= 0x04070107
#define SETMIN6 topMask |= 0x00000007; bottomMask |= 0x04070507
#define SETMIN7 topMask |= 0x00000007; bottomMask |= 0x01010101
#define SETMIN8 topMask |= 0x00000007; bottomMask |= 0x05070507
#define SETMIN9 topMask |= 0x00000007; bottomMask |= 0x05070107


// Some words for the menu system
#define RESET topMask |= 0x101010; bottomMask |= 0x10100000
#define SET topMask |= 0x10; bottomMask |= 0x10100000
#define IGHT bottomMask |= 0x0f000000
// #define HIGHT topMask |= 0x800; bottomMask |= 0x0f000000
#define NH topMask |= 0x8000800;
#define END topMask |= 0x19000000


// Brigthness level indicators for night
#define NIGHTINT1 bottomMask |= 0x00000080
#define NIGHTINT2 bottomMask |= 0x00008080
#define NIGHTINT3 bottomMask |= 0x00808080
#define NIGHTINT4 bottomMask |= 0x80808080
#define NIGHTINT5 topMask |= 0x00000080; bottomMask |= 0x80808080
#define NIGHTINT6 topMask |= 0x00008080; bottomMask |= 0x80808080
#define NIGHTINT7 topMask |= 0x00808080; bottomMask |= 0x80808080
#define NIGHTINT8 topMask |= 0x80808080; bottomMask |= 0x80808080

// Brigtness level indicators for day
#define DAYINT1 bottomMask |= 0x00000001
#define DAYINT2 bottomMask |= 0x00000101
#define DAYINT3 bottomMask |= 0x00010101
#define DAYINT4 bottomMask |= 0x01010101
#define DAYINT5 topMask |= 0x00000001; bottomMask |= 0x01010101
#define DAYINT6 topMask |= 0x00000101; bottomMask |= 0x01010101
#define DAYINT7 topMask |= 0x00010101; bottomMask |= 0x01010101
#define DAYINT8 topMask |= 0x01010101; bottomMask |= 0x01010101

// Hour setting for night start
#define NIGHTHOUR1 bottomMask |= 0x00000080
#define NIGHTHOUR2 bottomMask |= 0x000000C0
#define NIGHTHOUR3 bottomMask |= 0x000080C0
#define NIGHTHOUR4 bottomMask |= 0x0000C0C0
#define NIGHTHOUR5 bottomMask |= 0x8000C0C0
#define NIGHTHOUR6 bottomMask |= 0xC000C0C0
#define NIGHTHOUR7 bottomMask |= 0xC000C0C0; topMask |= 0x00000080
#define NIGHTHOUR8 bottomMask |= 0xC000C0C0; topMask |= 0x000000C0
#define NIGHTHOUR9 bottomMask |= 0xC000C0C0; topMask |=  0x008000C0
#define NIGHTHOUR10 bottomMask |= 0xC000C0C0; topMask |= 0x00C000C0
#define NIGHTHOUR11 bottomMask |= 0xC000C0C0; topMask |= 0x80C000C0
#define NIGHTHOUR12 bottomMask |= 0xC000C0C0; topMask |= 0xC0C000C0
// A indicates AM, nothing for PM
#define NIGHTHOURAM topMask |= 0x200000


// Hour setting for day start
#define DAYHOUR1 bottomMask |= 0x00000002
#define DAYHOUR2 bottomMask |= 0x00000003
#define DAYHOUR3 bottomMask |= 0x00000203
#define DAYHOUR4 bottomMask |= 0x00000303
#define DAYHOUR5 bottomMask |= 0x02000303
#define DAYHOUR6 bottomMask |= 0x03000303
#define DAYHOUR7 bottomMask |= 0x03000303; topMask |= 0x00000002
#define DAYHOUR8 bottomMask |= 0x03000303; topMask |= 0x00000003
#define DAYHOUR9 bottomMask |= 0x03000303; topMask |= 0x00020003
#define DAYHOUR10 bottomMask |= 0x03000303; topMask |= 0x00030003
#define DAYHOUR11 bottomMask |= 0x03000303; topMask |= 0x02030003
#define DAYHOUR12 bottomMask |= 0x03000303; topMask |= 0x03030003
// A indicates AM, nothing for PM
#define DAYHOURAM topMask |= 0x000400

// define pins
#define NEOPIN 6  // connect to DIN on NeoMatrix 8x8
#define STATUSLED 13 // LED on D13



// brightness based on time of day- could try warmer colors at night?

// Defaults

// Brightness, 1-8
#define DAYBRIGHTNESS 5
#define NIGHTBRIGHTNESS 2



// cutoff times for day / night brightness. feel free to modify.
#define MORNINGCUTOFF 7  // when does daybrightness begin?   7am
#define NIGHTCUTOFF   22 // when does nightbrightness begin? 10pm


// define delays
#define FLASHDELAY 250  // delay for startup "flashWords" sequence
#define SHIFTDELAY 50   // controls color shifting speed

#define MAXINTENSITY 100 // Set maximum/minimum intensities, 0 - 255, 1-8 value is mapped to this
#define MINITNENSITY 5   // Adjust as necessary for light filters used


//RTC_DS3234 RTC(10); // Establish clock object
RTC_DS3231 RTC;
DateTime thetime; // Holds current clock time

int j;   // an integer for the color shifting effect

#define HOURPIN 8
#define MINPIN 7

Bounce buttonHour = Bounce(); 
Bounce buttonMinute = Bounce();

// Parameter 1 = number of pixels in strip
// Parameter 2 = Arduino pin number (most are valid)
// Parameter 3 = pixel type flags, add together as needed:
//   NEO_KHZ800  800 KHz bitstream (most NeoPixel products w/WS2812 LEDs)
//   NEO_KHZ400  400 KHz (classic 'v1' (not v2) FLORA pixels, WS2811 drivers)
//   NEO_GRB     Pixels are wired for GRB bitstream (most NeoPixel products)
//   NEO_RGB     Pixels are wired for RGB bitstream (v1 FLORA pixels, not v2)
//Adafruit_NeoPixel matrix = Adafruit_NeoPixel(64, NEOPIN, NEO_GRB + NEO_KHZ800);

// configure for 8x8 neopixel matrix
Adafruit_NeoMatrix matrix = Adafruit_NeoMatrix(8, 8, NEOPIN,
                            NEO_MATRIX_TOP  + NEO_MATRIX_LEFT +
                            NEO_MATRIX_ROWS + NEO_MATRIX_PROGRESSIVE,
                            NEO_GRB         + NEO_KHZ800);
                            
int minuteButtonState;
int hourButtonState;
unsigned long buttonPressTimeStamp;
unsigned long firstButtonPressTimeStamp;
unsigned long setModeTimeStamp; // Used for set mode delays, use only within current function

bool timeValid = true; // becomes false if battery died and default time is used

byte dayIntensity = DAYBRIGHTNESS; // 1-8 setting
byte nightIntensity = NIGHTBRIGHTNESS; // 1-8 setting

byte nightStart = NIGHTCUTOFF;
byte dayStart = MORNINGCUTOFF;

void setup() {
  // put your setup code here, to run once:
  
  //Serial for debugging
  //Serial.begin(9600);
    
  // set pinmodes
  pinMode(NEOPIN, OUTPUT);
  pinMode(STATUSLED, OUTPUT); // Status LED
  delay(100);
  
  // time setting pins

  
  buttonHour.attach(HOURPIN, INPUT_PULLUP);
  buttonHour.interval(25);

  
  buttonMinute.attach(MINPIN, INPUT_PULLUP);
  buttonMinute.interval(25);

  matrix.begin();
  matrix.setBrightness(realIntensity(dayIntensity));
  matrix.fillScreen(0); // Initialize all pixels to 'off'
  matrix.show(); 


 // Holding either button for several seconds (5s) will clear settings in eeprom
 //  This will cause eeprom will checksum will fail and default values saved
  if(digitalRead(MINPIN) == LOW || digitalRead(HOURPIN) == LOW ){
      setModeTimeStamp = millis();
      delay(100);

      while(digitalRead(MINPIN) == LOW || digitalRead(HOURPIN) == LOW ){

        delay(100);
        
        if(millis() - setModeTimeStamp > 5000){ // show reset message once time greater than 5 seconds
        RESET;
        applyMaskColor(255,0,0);
       
        }
      }

      if(millis() - setModeTimeStamp > 5000){
        // button held 5 seconds at power on, reset eeprom. Checksum will become invalid and defaults saved later.
       // RESET;
       // applyMaskColor(255,0,0);
        doReset();
        delay(1000);
        }
  }

  

  
  // start clock
  Wire.begin(); // Begin I2C



  if (! RTC.begin()) {
    // Clock not working
    while (1);
  }

  
 if (RTC.lostPower())
 {
    // RTC is NOT running!
    // following line sets the RTC to the date & time this sketch was compiled
    // RTC.adjust(DateTime(__DATE__, __TIME__));
    RTC.adjust(DateTime(2021, 1, 1, 0, 0, 0));  // Load a default time

    timeValid = false; // battery died, time is not set, promt to set
    setModeTimeStamp = millis(); // reuse variable, used to flash set message
   
  
  }

  //Load settings from eeprom. If it fails, use teh defaults (already set above) and save them.
  if(! eepromLoad() ){
    eepromSave(); // eeprom data was invalid, save the current values, which are the defaults.
    
    }
    
}

void loop() {

  thetime = RTC.now();

// Check if button pressed, show menu/setings

  bool changed = buttonMinute.update() | buttonHour.update();
  if (changed && ( buttonMinute.read() == LOW || buttonHour.read() == LOW) ){
    
    // Entering set mode....
    digitalWrite(STATUSLED, HIGH);// LED Solid during setting modes
    
    matrix.setBrightness(realIntensity(dayIntensity));
    int option = 0; // Default optino 0 which is set, will be selected if time is not valid.

///////////////////////////////


    if(timeValid){
      showOption(option);
      
      setModeTimeStamp = millis();
  
      while( millis() - setModeTimeStamp < 2000 ){
  
        if ( (buttonMinute.update() && buttonMinute.read() == LOW) || (buttonHour.update() && buttonHour.read() == LOW) ){
  
            option++;
            if(option > 3){
              option = 0;
            }
  
            showOption(option);
            setModeTimeStamp = millis();
           }
      }

      // got an option, show it and do it
      // Show Selected option
      showOption(option);
      delay(150);
      applyMaskColor(0,0,0);
      delay(150);
      showOption(option);
      delay(150);
      applyMaskColor(0,0,0);
      delay(150);    

// display briefly or flash

    doOption(option);

////////////////////////////////////////////////////

    }else{ //Time is not valid, go straing to set - option will be 0
      doOption(option);
    }
    digitalWrite(STATUSLED, LOW);
    // leaving set mode
  }


  
  thetime = thetime.unixtime() + 150; // Show the time slightly in the future
 
  adjustBrightness();

  if(timeValid){
    displayTime();
    // flash the status led once per second while running normally
    if(millis() - setModeTimeStamp < 1000){
      digitalWrite(STATUSLED, HIGH);

    } else if(millis() - setModeTimeStamp < 2000) { 
        digitalWrite(STATUSLED, LOW);

    } else {
      setModeTimeStamp = millis();
      }

    
  } else {  // Flash set message if the time is not valid (dead battery)

    if(millis() - setModeTimeStamp < 1000){
     SET;
      matrix.setBrightness(realIntensity(dayIntensity)); // Make sure it can be seen
      applyMaskColor(255,0,0);
      digitalWrite(STATUSLED, HIGH);

    } else if ( millis() - setModeTimeStamp < 2000 ) {
      applyMaskColor(0,0,0);
      digitalWrite(STATUSLED, LOW);
      } else {
        setModeTimeStamp = millis();
      }
    }

}
